#!/usr/bin/env python

import glob, time, sys
from os.path import getmtime
from piglib import lookupDef, pigIndex
import Image, ImageMath
from PIL.ExifTags import TAGS
 
def get_exif(filename):
    ret = {}
    i = Image.open(filename)
    info = i._getexif()
    for tag, value in info.items():
        decoded = TAGS.get(tag, tag)
        ret[decoded] = value
    return ret

def isoTime(t):
	return time.strftime("%Y-%m-%dT%H:%M:%S", t)

def imageCtime(filename):
	exif=get_exif(filename)
	return lookupDef(exif, "DateTimeOriginal", lookupDef(exif, "DateTime", lookupDef(exif, "DateTimeDigitized", isoTime(time.localtime()))))
	
def interp(a, b, r):
	"Interpolate r proportion of the way between two images"
	expr="a+(b-a)*%f" % r
	aa=a.convert("RGB").split()
	bb=b.convert("RGB").split()
	ret=aa
	for n in range(len(aa)):
		r=ImageMath.eval(expr, a=aa[n].convert("F"), b=bb[n].convert("F"))
		ret[n].paste( r )
	return Image.merge("RGB", ret)
	
def main():
	globfilter='in/*.jpg'
	if len(sys.argv)>1:
		globfilter=sys.argv[1]
	noframes=10
	if len(sys.argv)>2:
		noframes=int(sys.argv[2])
	outbase="out"
	if len(sys.argv)>3:
		outbase=sys.argv[3]
	
	images=[ (f, getmtime(f)) for f in glob.glob(globfilter) ]
	images=sorted(images, key=lambda x: x[1])
	
	minsrctime=images[0][1]
	maxsrctime=images[-1][1]
	
	print "Found list of images: \n%s" % "\n".join( [ "(%s,%s)" % (f[0],f[1]) for f in images ])
	print "Computing %d frames" % noframes
	
	idx=oidx=0
	
	for n in range(noframes):
		t=(maxsrctime-minsrctime)*float(n)/noframes+minsrctime
		idx=pigIndex( [ x[1] for x in images ], lambda n: t<n ) -1
		fn1=images[idx][0]
		fn2=images[idx+1][0]
		
		if (idx!=oidx) or idx==0:		# only reload images on change
			im1=Image.open(fn1).convert("RGB")
			im2=Image.open(fn2).convert("RGB")
			oidx=idx
		r=float(t-images[idx][1])/(images[idx+1][1]-images[idx][1])
		print "Frame %d, t=%f images %s,%s r=%f" % (n,t, fn1, fn2, r)
		out=interp(im1, im2, r)
		out.convert("RGB").save("%s/result-%04d.jpg" % (outbase,n))
	
if __name__=="__main__":
	main()
